/*----------------------

   GATE - Geant4 Application for Tomographic Emission 
   OpenGATE Collaboration 
     
   Giovanni Santin <giovanni.santin@cern.ch> 
   Daniel Strul <daniel.strul@iphe.unil.ch> 
   Luc Simon <luc.simon@iphe.unil.ch> 
     
   Copyright (C) 2002 UNIL/IPHE, CH-1015 Lausanne 

This software is distributed under the terms 
of the GNU Lesser General  Public Licence (LGPL) 
See GATE/LICENSE.txt for further details 
----------------------*/

/*!
  \file GateGeneralParticleSourceMessenger.hh
  
  $Log: GateGeneralParticleSourceMessenger.hh,v $
  Revision 1.3  2002/10/11 14:52:38  lsimon
  Bug fix: the destruction of the UI directory is not left to the base-class GateMessenger, to avoid a double-attempt of destruction

  Revision 1.2  2002/08/11 15:33:24  dstrul
  Cosmetic cleanup: standardized file comments for cleaner doxygen output


  \brief Class GateGeneralParticleSourceMessenger
  \brief By Giovanni.Santin@cern.ch
  \brief $Id: GateGeneralParticleSourceMessenger.hh,v 1.3 2002/10/11 14:52:38 lsimon Exp $
*/
///////////////////////////////////////////////////////////////////////////////
// A large part of the code below is a direct copy of the content of the G4Tokenizer class
// from Geant4.4.0. It is thus the intellectual property of the Geant4 collaboration
// and is submitted to their disclaimer
///////////////////////////////////////////////////////////////////////////////
//
// Class Description:
//
// The function of the GateGeneralParticleSourceMessenger is to allow the user to
// enter commands either in interactive command line mode or through macros to
// control the G4GeneralParticleSource. The GateGeneralParticleSourceMessenger
// class is based on G4ParticleGunMessenger.
//
///////////////////////////////////////////////////////////////////////////////
//
// MEMBER FUNCTIONS
// ----------------
//
// GateGeneralParticleSourceMessenger(G4GeneralParticleSource *fPtclGun)
//     Constructor:  Sets up commands.
//
// ~GateGeneralParticleSourceMessenger()
//     Destructor:  Deletes commands.
//
// void SetNewValue(G4UIcommand *command, G4String newValues)
//     Uses the appropriate methods in the G4GeneralParticleSource to carry out
//     the user commands.
//
// G4String GetCurrentValue(G4UIcommand *command)
//     Allows the user to retrieve the current values of parameters.
//     Not implemented yet.
//
///////////////////////////////////////////////////////////////////////////////
//
// CHANGE HISTORY
// --------------
//
// Version 1.0, 28 February 2000, C Ferguson, Created.
//
// Version 1.1, 19 October 2000, Modified to inherit from G4VPrimaryGenerator.
// New name at the request of M. Asai.
//
///////////////////////////////////////////////////////////////////////////////
//
#ifndef GateGeneralParticleSourceMessenger_h
#define GateGeneralParticleSourceMessenger_h 1

#include "G4UImessenger.hh"
#include "globals.hh"
//#include "UIcmdWithNucleusAndUnit.hh"

#include "GateMessenger.hh"

class G4ParticleTable;
class G4UIcommand;
class G4UIdirectory;
class G4UIcmdWithoutParameter;
class G4UIcmdWithAString;
class G4UIcmdWithADoubleAndUnit;
class G4UIcmdWith3Vector;
class G4UIcmdWith3VectorAndUnit;
class G4UIcmdWithAnInteger;
class G4UIcmdWithADouble;
class G4UIcmdWithABool;
class G4UIcmdWithoutParameter;

//class G4GeneralParticleSource;
class GateGeneralParticleSource;

class GateGeneralParticleSourceMessenger: public GateMessenger
{
public:
  //  GateGeneralParticleSourceMessenger(G4GeneralParticleSource *fPtclGun);
  GateGeneralParticleSourceMessenger(GateGeneralParticleSource *fPtclGun);
  ~GateGeneralParticleSourceMessenger();
    
  void SetNewValue(G4UIcommand *command, G4String newValues);
  //    Identifies the command which has been invoked by the user, extracts the
  //    parameters associated with that command (held in newValues), and uses
  //    these values with the appropriate member function of G4GeneralParticleSource.
  G4String GetCurrentValue(G4UIcommand *command);

private:
  void IonCommand(G4String newValues);

private:
  //  G4GeneralParticleSource *fParticleGun;
  GateGeneralParticleSource *fParticleGun;
  G4ParticleTable *particleTable;
  G4String histtype;
    
private: //commands

  G4UIcmdWithAString         *typeCmd;
  G4UIcmdWithAString         *shapeCmd;
  G4UIcmdWith3VectorAndUnit  *centreCmd;
  G4UIcmdWith3Vector         *posrot1Cmd;
  G4UIcmdWith3Vector         *posrot2Cmd;
  G4UIcmdWithADoubleAndUnit  *halfxCmd;
  G4UIcmdWithADoubleAndUnit  *halfyCmd;
  G4UIcmdWithADoubleAndUnit  *halfzCmd;
  G4UIcmdWithADoubleAndUnit  *radiusCmd;
  G4UIcmdWithADoubleAndUnit  *radius0Cmd;
  G4UIcmdWithADoubleAndUnit  *possigmarCmd;
  G4UIcmdWithADoubleAndUnit  *possigmaxCmd;
  G4UIcmdWithADoubleAndUnit  *possigmayCmd;
  G4UIcmdWithADoubleAndUnit  *paralpCmd;
  G4UIcmdWithADoubleAndUnit  *partheCmd;
  G4UIcmdWithADoubleAndUnit  *parphiCmd;  
  G4UIcmdWithAString         *confineCmd;         

  G4UIcmdWithAString         *angtypeCmd;
  G4UIcmdWith3Vector         *angrot1Cmd;
  G4UIcmdWith3Vector         *angrot2Cmd;
  G4UIcmdWithADoubleAndUnit  *minthetaCmd;
  G4UIcmdWithADoubleAndUnit  *maxthetaCmd;
  G4UIcmdWithADoubleAndUnit  *minphiCmd;
  G4UIcmdWithADoubleAndUnit  *maxphiCmd;
  G4UIcmdWithADoubleAndUnit  *angsigmarCmd;
  G4UIcmdWithADoubleAndUnit  *angsigmaxCmd;
  G4UIcmdWithADoubleAndUnit  *angsigmayCmd;
  G4UIcmdWithABool           *useuserangaxisCmd;
  G4UIcmdWithABool           *surfnormCmd;

  G4UIcmdWithAString         *energytypeCmd;
  G4UIcmdWithADoubleAndUnit  *eminCmd;
  G4UIcmdWithADoubleAndUnit  *emaxCmd;
  G4UIcmdWithADoubleAndUnit  *monoenergyCmd;
  G4UIcmdWithADoubleAndUnit  *engsigmaCmd;
  G4UIcmdWithADouble         *alphaCmd;
  G4UIcmdWithADouble         *tempCmd;
  G4UIcmdWithADouble         *ezeroCmd;
  G4UIcmdWithADouble         *gradientCmd;
  G4UIcmdWithADouble         *interceptCmd;
  G4UIcmdWithoutParameter    *calculateCmd;
  G4UIcmdWithABool           *energyspecCmd;
  G4UIcmdWithABool           *diffspecCmd;

  G4UIcmdWith3Vector         *histpointCmd;
  G4UIcmdWithAString         *histnameCmd;
  G4UIcmdWithAString         *arbintCmd;

  G4UIcmdWithAString         *resethistCmd;

  G4UIcmdWithAnInteger       *verbosityCmd;

  // below are commands from G4ParticleGun

  G4UIcommand                *ionCmd;

  G4UIcmdWithAString         *particleCmd;
  G4UIcmdWithADoubleAndUnit  *timeCmd;
  G4UIcmdWith3Vector         *polCmd;
  G4UIcmdWithAnInteger       *numberCmd;

  G4UIcmdWith3VectorAndUnit  *positionCmd;
  G4UIcmdWith3Vector         *directionCmd;
  G4UIcmdWithADoubleAndUnit  *energyCmd;
  G4UIcmdWithoutParameter    *listCmd;


  private: // for ion shooting
    G4bool   fShootIon; 
    G4int    fAtomicNumber;
    G4int    fAtomicMass;
    G4int    fIonCharge;
    G4double fIonExciteEnergy;

};

#endif
